@@ -3,7 +3,7 @@
 #
 # Copyright (C) 2008 Hasiotis Nikos
 #
-# ID: $Id: Base.pm 45 2008-11-25 00:37:50Z hasiotis $
+# ID: $Id: Base.pm 51 2009-01-08 09:17:46Z hasiotis $
 
 package EAFDSS::Base;
 
@@ -11,17 +11,18 @@ use 5.006001;
 use strict;
 use warnings;
 use Carp;
+use Class::Base;
 use Data::Dumper;
 
 use base qw ( Class::Base );
 
-our($VERSION) = '0.11';
+our($VERSION) = '0.12';
 
 sub init {
 	my($self, $config) = @_;
 
 	if (! exists $config->{DIR}) {
-		return $self->error("You need to provide the DIR to save the singatures!");
+		return $self->error("You need to provide the DIR to save the signnatures!");
 	} else {
 		$self->{DIR} = $config->{DIR};
 	}
@@ -48,6 +49,16 @@ sub Sign {
 			return $self->error($replySignDir);
 		}
 
+                $self->debug(  "  Checking file [%s]", $fname);
+                open(FH, $fname);
+		my($invalid) = $self->_checkCharacters(*FH);
+                close(FH);
+
+                if ($invalid)  {
+			$self->debug("  File contains invalid characters [%s]", $fname);
+			return $self->error(64+0x10);
+		}
+
                 $self->debug(  "  Signing file [%s]", $fname);
                 open(FH, $fname);
                 ($reply, $totalSigns, $dailySigns, $date, $time, $nextZ, $sign) = $self->PROTO_GetSign(*FH);
@@ -71,6 +82,21 @@ sub Sign {
 
 }
 
+sub _checkCharacters {
+        my($self) = shift @_;
+	my($fh)   = shift @_;
+	
+	my($c);
+	while (read($fh, $c, 1)) {
+		if (grep $_ == ord($c), qw/0 1 2 3 4 5 6 7 8 11 14 15 16 17 18 19 20 21 22 23 24 25 27 28 29 30 31 127 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 173 210 255/ ) {
+                	$self->debug("  Found invalid character [%d]", ord($c));
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 sub Status {
         my($self) = shift @_;
 
@@ -147,6 +173,22 @@ sub Info {
 	}
 }
 
+sub Query {
+        my($self) = shift @_;
+
+        $self->debug("Query for devices");
+	my($reply, $devices) = $self->PROTO_Query();
+	if ($reply == 0) {
+		if ($devices) {
+        		return $devices;
+		} else {
+			return $self->error(64+0x05);
+		}
+	} else {
+		return $self->error($reply);
+	}
+}
+
 sub GetHeaders {
         my($self) = shift @_;
 
@@ -422,7 +464,7 @@ Read EAFDSS on how to use the module.
 
 =head1 VERSION
 
-This is version 0.10.
+This is version 0.12.
 
 =head1 AUTHOR
 
